︠1e3ad33e-59c7-4dbc-b10d-f48b375c53dai︠
%hide
# By Graham Enos (based on the work of Nick Alexander (based on the work of
# Marshall Hampton))

var('x')

@interact
def riemann_rects(f = input_box(default = sin(x), type = SR,),
    a = input_box(default = "0", type = str),
    b = input_box(default = "pi/2", type = str),
    n = slider(1, 20, 1, default=4, label="n"),
    endpoint_rule = selector(['Left', 'Right', 'Midpoint', 'Trapezoid'],
        nrows=1, label="Endpoint rule")):
            
# General Setup
    a, b = sage_eval(a), sage_eval(b)
    t = sage.calculus.calculus.var('t')
    func = fast_callable(f(x=t), vars=[t])
    dx = (b-a)/(n)
    rects = Graphics()

# Exact answer (from Sage)
    exact_answer = integral(func(t),t,a,b)

# Print only a few digits of precision
    def cap(x):
        return RealField(20)(x)

# Lists of x and y coordinates, depending on endpoint_rule
    if endpoint_rule == 'Left':
        xs = [q*dx + a for q in range(n)]
    elif endpoint_rule == 'Right':
        xs = [(q+1)*dx + a for q in range(n)]
    elif endpoint_rule == 'Midpoint':
        xs = [q*dx + a + dx/2 for q in range(n)]
    else:
        xs = [q*dx + a for q in range(n+1)]
    ys = [func(x) for x in xs]
         
################################################
# The rest depends on rectangles vs trapezoids #
################################################

### Rectangles
    if endpoint_rule in ['Left', 'Right', 'Midpoint']:

# Constructing rectangles
        for q in range(n):
            xm = q*dx + dx/2 + a
            x = xs[q]
            y = ys[q]
            rects += line([[xm-dx/2,0],[xm-dx/2,y],[xm+dx/2,y],[xm+dx/2,0]],
                    rgbcolor = (1,0,0))

# Plotting: find max_y and min_y to determine height of plot
        min_y = min(0, find_minimum_on_interval(func,a,b)[0])
        max_y = max(0, find_maximum_on_interval(func,a,b)[0])
        show(plot(func,a,b,thickness=2) + rects, xmin = a, xmax = b, ymin =
                min_y, ymax = max_y)

# Pretty printing setup for sum and numerical values
        sum_html = "%s • [%s]" % (cap(dx), ' + '.join([" f(%s)"
            % cap(i) for i in xs]))
        num_html = "%s • [%s]" % (cap(dx), ' + '.join([str(
                cap(i)) for i in ys]))

# Estimated answer via Riemann Sums
        estimated_answer = dx * sum([ys[q] for q in range(n)])

# Pretty printing, using \LaTeX
        html(r'''
            Integral = %s
            
            Sum from 1 to %s of f(x_k) Δx
            = %s
            = %s
            = %s
            
            Absolute Error = %s
        ''' % (exact_answer, n, sum_html, num_html, estimated_answer.n(),
            abs(exact_answer - estimated_answer).n()))

### Trapezoids
    else:

# Constructing trapezoids
        for q in range(n):
            xL, xR = xs[q], xs[q+1]
            yL, yR = ys[q], ys[q+1]
            rects += line([[xL,0],[xL,yL],[xR,yR],[xR,0]], rgbcolor = (1,0,0))

# Plotting: find max_y and min_y to determine height of plot
        min_y = min(0, find_minimum_on_interval(func,a,b)[0])
        max_y = max(0, find_maximum_on_interval(func,a,b)[0])
        show(plot(func,a,b,thickness=2) + rects, xmin = a, xmax = b, ymin =
                min_y, ymax = max_y)

# Pretty printing setup for sum and numerical values; depends on endpoint_rule
        sum_html = "%s/2 • [%s]" % (cap(dx), ' + '.join
                (["[f(%s) + f(%s)] " % (cap(xs[i]), cap(xs[i+1])) for i in
                    range(n)]))
        num_html = "%s/2 • [%s]" % (cap(dx), ' + '.join
                ([str(cap(i)) for i in ys]))

# Estimated answer via Riemann Sums
        estimated_answer = dx/2 * sum(ys[i] + ys[i+1] for i in range(n))

# Pretty printing, using LaTeX
        html(r'''
            Integral = %s
            
            Sum from 1 to %s of [f(x_{k-1}) + f(x_k)]/2 • Δx
            = %s
            = %s
            = %s
            
            Absolute Error = %s
        ''' % (exact_answer, n, sum_html, num_html, estimated_answer.n(),
            abs(exact_answer - estimated_answer).n()))
︡638d1183-81da-4ada-9fdf-ec3f319a4e56︡{"html": ""}︡
